# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py
# -*- encoding: utf-8 -*-

from acrylamid.views import View
from acrylamid.utils import render, mkfile, joinurl

from jinja2 import Template

filters = []
path = '/page/'
items_per_page = 10
paginating = True
enabled = True

class Index(View):
    
    def __init__(self, conf, env):
        pass
        
    def __call__(self, request):
        """Creates nicely paged listing of your posts.  First page is the
        index.hml used to have this nice url: http://yourblog.com/ with a recent
        list of your (e.g. summarized) Posts. Other pages are enumerated to /page/n+1

        required:
        items_per_page -- posts displayed per page (defaults to 6)
        entry.html -- layout of Post's entry
        main.html -- layout of the website
        """
        conf = request['conf']
        env = request['env']
        entrylist = request['entrylist']
        ipp = items_per_page

        tt_entry = Template(env['tt_entry'])
        tt_main = Template(env['tt_main'])

        entrylist = [render(tt_entry, conf, env, entry, type="page") for entry in entrylist]

        for i, mem in enumerate([entrylist[x*ipp:(x+1)*ipp] for x in range(len(entrylist)/ipp+1)]):

            html = render(tt_main, conf, env, type='page', page=i+1,
                        entrylist='\n'.join(mem), num_entries=len(entrylist),
                        items_per_page=items_per_page)
            directory = conf['output_dir']
            if i > 0:
                directory = joinurl(conf['output_dir'], (path + str(i+1)))
                
            p = joinurl(directory, 'index.html')
            mkfile(html, {'title': '/' if i==0 else (path+str(i+1))}, p)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import logging
import os
import re
import fnmatch
import time
from datetime import datetime
from collections import defaultdict

from acrylamid.utils import FileEntry
from jinja2 import Template

log = logging.getLogger('acrylamid.core')
    

def start(request):
    """this loads entry.html and main.html into data and does a little
    preprocessing: {{ include: identifier }} will be replaced by the
    corresponding identifier in request._env if exist else empty string."""
    
    conf = request['conf']
    env = request['env']
    
    regex = re.compile('{{\s*include:\s+(\w+)\s*}}')
    entry = open(os.path.join(conf['layout_dir'], 'entry.html')).read()
    main = open(os.path.join(conf['layout_dir'], 'main.html')).read()
    
    d = defaultdict(str)
    d.update(env)
    env['tt_entry'] = re.sub(regex, lambda m: d[m.group(1)], entry)
    env['tt_main'] = re.sub(regex, lambda m: d[m.group(1)], main)
    
    return request
    

def handle(request):
    """This is the acrylamid handle:
        - generate filelist
        - sort filelist by date"""
        
    conf = request['conf']

    # generate a list of entries
    request['entrylist'] = filelist(request)

    for i,entry in enumerate(request['entrylist']):
        # convert mtime timestamp or `date:` to localtime (float), required for sort
        if isinstance(entry.date, basestring):
            timestamp = time.mktime(time.strptime(entry.date,
                conf.get('strptime', '%d.%m.%Y, %H:%M')))
            entry.date = datetime.fromtimestamp(timestamp)
        else:
            log.warn("using mtime from %s" % entry)
    
    request['entrylist'].sort(key=lambda k: k.date, reverse=True)
    return request


def filelist(request):
    """This is the default handler for getting entries.  It takes the
    request object in and figures out which entries bases on the default
    behavior that we want to show and generates a list of EntryBase
    subclass objects which it returns.
    
    Arguments:
    args -- dict containing the incoming Request object
    
    Returns the content we want to render"""
    
    conf = request['conf']
    
    filelist = []
    for root, dirs, files in os.walk(conf['entries_dir']):
        for file in files:
            path = os.path.join(root, file)
            fn = filter(lambda p: fnmatch.fnmatch(path, os.path.join(conf['entries_dir'], p)),
                        conf.get('entries_ignore', []))
            if not fn:
                filelist.append(path)
    
    entrylist = [FileEntry(e) for e in filelist]
    return entrylist


def format(request):
    """Applies:
        - cb_preformat
        - cb_format
        - cb_postformat"""
    
    # entry = request['entry']
    # 
    # entry = tools.run_callback(
    #         'preformat',
    #         {'entry': entry, 'config': request['config']},
    #         defaultfunc=preformat)['entry']
    # 
    # entry = tools.run_callback(
    #         'format',
    #         {'entry': entry, 'config': request['config']},
    #         defaultfunc=format)['entry']
    #     
    # entry = tools.run_callback(
    #         'postformat',
    #         {'entry': entry, 'config': request['config']})['entry']
    
    return request
    
def preformat(request):
    """joins content from file (stored as list of strings)"""
    
    entry = request['entry']
    entry['body'] = ''.join(entry['story'])
    return request

def format(request):
    """Apply markup using Post.parser."""
    
    entry = request['entry']
    conf = request['config']
    
    parser = entry.get('parser', conf.get('parser', 'plain'))
    
    if parser.lower() in ['markdown', 'mkdown', 'md']:
        from markdown import markdown
        entry['body'] = markdown(entry['body'],
                        extensions=['codehilite(css_class=highlight)'])
        return request
    
    elif parser.lower() in ['restructuredtest', 'rst', 'rest']:                
        from docutils import nodes
        from docutils.parsers.rst import directives, Directive
        from docutils.core import publish_parts
        
        # Set to True if you want inline CSS styles instead of classes
        INLINESTYLES = False
        
        from pygments.formatters import HtmlFormatter
        
        # The default formatter
        DEFAULT = HtmlFormatter(noclasses=INLINESTYLES)
        
        
        from docutils import nodes
        from docutils.parsers.rst import directives, Directive
        
        from pygments import highlight
        from pygments.lexers import get_lexer_by_name, TextLexer
        
        class Pygments(Directive):
            """ Source code syntax hightlighting.
            """
            required_arguments = 1
            optional_arguments = 0
            final_argument_whitespace = True
            option_spec = dict([(key, directives.flag) for key in {}])
            has_content = True
    
            def run(self):
                self.assert_has_content()
                try:
                    lexer = get_lexer_by_name(self.arguments[0])
                except ValueError:
                    # no lexer found - use the text one instead of an exception
                    lexer = TextLexer()
                # take an arbitrary option if more than one is given
                formatter = self.options and VARIANTS[self.options.keys()[0]] or DEFAULT
                parsed = highlight(u'\n'.join(self.content), lexer, formatter)
                return [nodes.raw('', parsed, format='html')]
        
        directives.register_directive('sourcecode', Pygments)
        initial_header_level = 1
        transform_doctitle = 0
        settings = {
            'initial_header_level': initial_header_level,
            'doctitle_xform': transform_doctitle
            }
        parts = publish_parts(
            entry['body'], writer_name='html', settings_overrides=settings)
        entry['body'] = parts['body'].encode('utf-8')
        return request
    else:
        return request

def prepare(request):
    """Sets a few required keys in FileEntry.
    
    required:
    safe_title -- used as directory name
    url -- permalink
    id -- atom conform id: http://diveintomark.org/archives/2004/05/28/howto-atom-id
    """
    conf = request._config
    data = request._data
    
    for i, entry in enumerate(data['entry_list']):
        safe_title = tools.safe_title(entry['title'])
        url = '/' + str(entry.date.year) + '/' + safe_title + '/'
        id = 'tag:' + re.sub('https?://', '', conf.get('www_root', '')).strip('/') \
             + ',' + entry.date.strftime('%Y-%m-%d') + ':' \
             + '/' + str(entry.date.year) +  '/' + safe_title
        item = data['entry_list'][i]
        if not 'safe_title' in item:
            item['safe_title'] = safe_title
        if not 'url' in item:
            item['url'] = url
        if not 'id' in item:
            item['id'] = id
    
    return request



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
# 
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
# 
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of posativ <info@posativ.org>.

VERSION = "0.1.10-dev"
VERSION_SPLIT = tuple(VERSION.split('-')[0].split('.'))

import sys
reload(sys); sys.setdefaultencoding('utf-8')

import os, logging, locale
from optparse import OptionParser, make_option, OptionGroup

from acrylamid import defaults
from acrylamid import filters
from acrylamid import views
from acrylamid.utils import check_conf, yamllike, ColorFormatter

log = logging.getLogger('acrylamid')
sys.path.insert(0, os.path.dirname(__package__))


class Acryl:
    """Main class for acrylamid functionality.  It handles initialization,
    defines default behavior, and also pushes the request through all
    steps until the output is rendered and we're complete."""
    
    def __init__(self):
        """Sets configuration and environment and creates the Request
        object"""
        
        usage = "usage: %prog [options] init\n" + '\n' \
                + "init     - initializes base structure\n" \
                + "gen      - rendering blog\n"
#                + "srv (-p) - serving on port p (8000) and auto-rendering changes\n"
        
        options = [
            make_option("-v", "--verbose", action="store_const", dest="verbose",
                              help="debug information", const=logging.DEBUG),
            make_option("-q", "--quit", action="store_const", dest="verbose",
                              help="be silent (mostly)", const=logging.WARN,
                              default=logging.INFO),
            make_option("-f", "--force", action="store_true", dest="force",
                              help="force overwriting", default=False),
            make_option("-c", "--config", dest="conf", help="alternate conf.yaml",
                              default="conf.yaml"),
            make_option("--version", action="store_true", dest="version",
                               help="print version details", default=False),                  
            ]
            
            
        parser = OptionParser(option_list=options, usage=usage)
        ext_group = OptionGroup(parser, "conf.yaml override")
        
        for key, value in yamllike(defaults.conf).iteritems():
            if key in ['views.', 'filters.']: continue
            ext_group.add_option('--'+key.replace('_', '-'), action="store",
                            dest=key, metavar=value, type=type(value) if type(value) != list else str)
                            
        parser.add_option_group(ext_group)
        (options, args) = parser.parse_args()
            
        console = logging.StreamHandler()
        console.setFormatter(ColorFormatter('%(message)s'))
        if options.verbose == logging.DEBUG:
            fmt = '%(msecs)d [%(levelname)s] %(name)s.py:%(lineno)s:%(funcName)s %(message)s'
            console.setFormatter(ColorFormatter(fmt))
        log = logging.getLogger('acrylamid')
        log.addHandler(console)
        log.setLevel(options.verbose)
        
        env = {'VERSION': VERSION, 'VERSION_SPLIT': VERSION_SPLIT}
        if options.version:
            print "acrylamid" + ' ' + env['VERSION']
            sys.exit(0)
        
        if len(args) < 1:
            parser.print_usage()
            sys.exit(1)
        
        # -- init -- #
        # TODO: acrylamid init --layout_dir=somedir to overwrite defaults
        
        if 'init' in args:
            if len(args) == 2:
                defaults.init(args[1], options.force)
            else:
                defaults.init(overwrite=options.force)
            sys.exit(0)
        
        try:
            conf = yamllike(open(options.conf).read())
        except OSError:
            log.critical('no config file found: %s. Try "acrylamid init".', options.conf)
            sys.exit(1)
        
        assert check_conf(conf)
                
        conf['acrylamid_name'] = "acrylamid"
        conf['acrylamid_version'] = env['VERSION']

        conf['output_dir'] = conf.get('output_dir', 'output/')
        conf['entries_dir'] = conf.get('entries_dir', 'content/')
        conf['layout_dir'] = conf.get('layout_dir', 'layouts/')
        
        conf.update(dict((k,v) for k,v in options.__dict__.iteritems() if v != None))

        # -- run -- #
                    
        if args[0] in ['gen', 'generate', 'render']:
            self.req = {'conf': conf, 'env': env, 'data': {}}
            
            self.run()
            # from filters.md import Markdown
            # md = Markdown()
            # print md('Hallo $$beta$$-Welt', [],  *['math'])
            #self.run(request)
                
    def initialize(self, request):
        """The initialize step further initializes the Request by
        setting additional information in the ``data`` dict,
        registering plugins, and entryparsers.
        """
        
        conf = request['conf']
        env = request['env']

        # initialize the locale, will silently fail if locale is not
        # available and uses system's locale
        try:
            locale.setlocale(locale.LC_ALL, conf.get('lang', ''))
        except (locale.Error, TypeError):
            # invalid locale
            locale.setlocale(locale.LC_ALL, '')
            log.warn("unsupported locale '%s', set to '%s'", conf['lang'], locale.getlocale()[0])
        conf['lang'] = locale.getlocale()

        if not conf.has_key('www_root'):
            log.warn('no `www_root` specified, using localhost:8000')
            conf['www_root'] = 'http://localhost:8000/'
        
        env['protocol'] = conf['www_root'][0:conf['www_root'].find('://')]
        # take off the trailing slash for base_url
        if conf['www_root'].endswith("/"):
            conf['www_root'] = conf['www_root'][:-1]

        # import and initialize plugins
        filters.initialize(conf.get("ext_dir", []), request['conf'], request['env'],
                              exclude=conf.get("ext_ignore", []),
                              include=conf.get("ext_include", []))
        views.initialize(conf.get("ext_dir", []), request['conf'], request['env'])

        ns = filters.__dict__
        exec(''.join(conf['filters.'])) in ns
        ns = views.__dict__
        exec(''.join(conf['views.'])) in ns
        
    def run(self):
        """This is the main loop for acrylamid.  This method will run
        the handle callback to allow registered handlers to handle
        the request. If nothing handles the request, then we use the
        ``_acrylamid_handler``.
        """
        request = self.req
        conf = self.req['conf']
        self.initialize(request)
        
        from acrylamid.core import start, handle
        from acrylamid.filters import get_filters, FilterList
        from acrylamid.views import get_views
        
        request = start(request)
        request = handle(request)
        
        filtersdict = get_filters()
        _views = get_views()
        
        for v in _views:
            log.debug(v)
            for i, entry in enumerate(request['entrylist']):
                if not v.__filters__: break
                
                log.debug(entry.filename)
                entry.content = entry.source
                entryfilters = entry.get('filters', [])
                if isinstance(entryfilters, basestring):
                    entryfilters = [entryfilters]
                viewsfilters = views.filters+getattr(views, v.__module__).filters
                
                _filters = FilterList()
                for f in entryfilters+viewsfilters:
                    x,y = f.split('+')[:1][0], f.split('+')[1:]
                    if filtersdict[x] not in _filters:
                        _filters.append((x, filtersdict[x], y))
                
                for i, f, args in _filters:
                    f.__dict__['__matched__'] = i
                    entry.content = f(entry.content, entry, *args)
            
            v(request)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2011 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid.py

import sys, os, re
import codecs
from datetime import datetime
from os.path import join, exists, getmtime, dirname
from time import gmtime
import logging

log = logging.getLogger('acrylamid.utils')
_slug_re = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.:]+')

try:
    import translitcodec
except ImportError:
    import unicodedata
    from string import maketrans
    from unicodedata import normalize
    translitcodec = None
    log.debug("no 'translitcodec' found, using NFKD algorithm")


class FileEntry:
    """This class gets it's data and metadata from the file specified
    by the filename argument"""
    
    # TODO: remap internally
    __map__ = {'tag': 'tags', 'filter': 'filters'}
    __keys__ = ['permalink', 'filters', 'author', 'draft', 'tags', 'date', 'title', 'content', 'lang']
    
    title = content = ''
    draft = False
    permalink = lang = None
    tags = filters = []
    
    def __init__(self, filename, encoding='utf-8', new=True):
        """Arguments:
        request -- the Request object
        filename -- the complete filename including path
        datadir --  the data dir"""
        
        self.date = datetime.fromtimestamp(os.path.getmtime(filename))
        self.filename = filename
        self.encoding = encoding
        self.parse()
        
    def __repr__(self):
        return "<fileentry f'%s'>" % self.filename
        
    @property
    def extension(self):
        return os.path.splitext(self.filename)[1][1:]
        
    @property
    def source(self):
        with codecs.open(self.filename, 'r', encoding=self.encoding) as f:
            return ''.join(f.readlines()[self._i:]).strip()
            
    @property
    def slug(self):
        """Generates an ASCII-only slug.  Borrowed from
        http://flask.pocoo.org/snippets/5/"""

        result = []
        for word in _slug_re.split(self.title.lower()):
            if translitcodec:
                word = word.encode('translit/long')
            else:
                word = normalize('NFKD', word).encode('ascii', 'ignore')
            if word and not word[0] in '-:':
                result.append(word)
        return unicode('-'.join(result))
        
    @property
    def permalink(self):
        # TODO: fix hard-coded slug
        return expand('/:year/:slug/', self)
        
    def get(self, key, default=None):
        return self.__dict__.get(key, default)
        
    def parse(self):
        """parsing yaml header and remember when content begins."""
        
        meta = []; i = 0
        with file(self.filename, 'r') as f:
            while True:
                line = f.readline()
                i += 1
                if i == 1 and not line.strip():
                    break
                elif i == 1 and line.startswith('---'):
                    pass
                elif i > 1 and not line.startswith('---'):
                    meta.append(line)
                else:
                    break
                
        self._i = i
        for key, value in yamllike(''.join(meta)).iteritems():
            if not hasattr(self, key):
                self.__dict__[key] = value
            elif key in self.__keys__:
                if isinstance(value, basestring):
                    self.__dict__[key] = unicode(value.strip('"'))
                else:
                    self.__dict__[key] = value
                    
    def keys(self):
        return filter(lambda k: hasattr(self, k), self.__keys__)
        
    def __getitem__(self, key):
        if key in self.__map__:
            return self.__dict__[self.__map__[key]]
        elif key in self.__keys__:
            return getattr(self, key)
        else:
            raise KeyError("%s has no such attribute '%s'" % (self, key))
            


class ColorFormatter(logging.Formatter):
    """Implements basic colored output using ANSI escape codes."""

    # -- and BOLD
    BLACK = '\033[1;30m%s\033[0m'
    RED = '\033[1;31m%s\033[0m'
    GREEN = '\033[1;32m%s\033[0m'
    YELLOW = '\033[1;33m%s\033[0m'
    GREY = '\033[1;37m%s\033[0m'
    RED_UNDERLINE = '\033[4;31m%s\033[0m'

    def __init__(self, fmt='[%(levelname)s] %(name)s: %(message)s', debug=False):
        logging.Formatter.__init__(self, fmt)
        self.debug = debug

    def format(self, record):

        keywords = {'skip': self.BLACK, 'create': self.GREEN, 'identical': self.BLACK,
                    'update': self.YELLOW, 'changed': self.YELLOW}
                    
        if record.levelno == logging.INFO:
            for item in keywords:
                if record.msg.startswith(item):
                    record.msg = record.msg.replace(item, ' '*2 + \
                                    keywords[item] % item.rjust(8))
        elif record.levelno >= logging.WARN:
            record.levelname = record.levelname.replace('WARNING', 'WARN')
            record.msg = ''.join([' '*2, self.RED % record.levelname.lower().rjust(8),
                                  '  ', record.msg])

        return logging.Formatter.format(self, record)


def check_conf(conf):
    """Rudimentary conf checking.  Currently every *_dir except
    `ext_dir` (it's a list of dirs) is checked wether it exists."""

    # directories

    for key, value in conf.iteritems():
        if key.endswith('_dir') and not key in ['ext_dir', ]:
            if os.path.exists(value):
                if os.path.isdir(value):
                    pass
                else:
                    log.error("'%s' must be a directory" % value)
                    sys.exit(1)
            else:
                os.mkdir(value)
                log.warning('%s created...' % value)

    return True
    
def yamllike(conf):
    
    conf = [line.strip() for line in conf.split('\n')
                if not line.startswith('#') and line.strip()]
    
    config = {}
    config['views.'] = []
    config['filters.'] = []
    for line in conf:
        try:
            key, value = [x.strip() for x in line.split(':', 1)]
        except ValueError:
            # do something
            log.warn('conf.yaml -> ValueError: %s' % line)
            continue
        
        if key.startswith('filters.'):
            config['filters.'].append(key.replace('filters.', '')+' = '+value+'\n')
        elif key.startswith('views.'):
            config['views.'].append(key.replace('views.', '')+' = '+value+'\n')
        else:
            if value == '':
                config[key] = None
            elif value.isdigit():
                config[key] = int(value)
            elif value.lower() in ['true', 'false']:
                 config[key] = True if value.capitalize() == 'True' else False
            elif value[0] == '[' and value[-1] == ']':
                config[key] = list([x.strip() for x in value[1:-1].split(',')])
            else:
                config[key] = value
    
    return config


def render(tt, *dicts, **kvalue):
    """helper function to merge multiple dicts and additional key=val params
    to a single environment dict used by jinja2 templating. Note, merging will
    first update dicts in given order, then (possible) overwrite single keys
    in kvalue."""

    env = {}
    for d in dicts:
        env.update(d)
    for key in kvalue:
        env[key] = kvalue[key]
    
    return tt.render(env)


def mkfile(content, entry, path, force=False):
    """Creates entry in filesystem. Overwrite only if content
    differs.

    Arguments:
    content -- rendered html
    entry -- FileEntry object
    path -- path to write
    force -- force overwrite, even nothing has changed (defaults to `False`)
    """

    if exists(dirname(path)) and exists(path):
        with file(path) as f:
            old = f.read()
        if content == old and not force:
            log.info("skip  '%s' is up to date" % entry['title'])
        else:
            f = open(path, 'w')
            f.write(content)
            f.close()
            log.info("changed  content of '%s'" % entry['title'])
    else:
        try:
            os.makedirs(dirname(path))
        except OSError:
            # dir already exists (mostly)
            pass
        f = open(path, 'w')
        f.write(content)
        f.close()
        log.info("create  '%s', written to %s" % (entry['title'], path))

    
def expand(url, entry):
    m = {':year': str(entry.date.year), ':month': str(entry.date.month),
         ':day': str(entry.date.day), ':slug': entry.slug}
    
    for val in m:
        url = url.replace(val, m[val])
    return url


def joinurl(*args):
    """joins multiple urls to one single domain without loosing root (first element)"""
    
    r = []
    for i, mem in enumerate(args):
        if i > 0:
            mem = mem.lstrip('/')
        r.append(mem)
    
    return join(*r)
